from time import perf_counter

from evaluation.export import write_row
from evaluation.T import (
    GasstationQueryRow,
    ChargeQueryRow,
    ClassicQueryRow,
    AStarQueryRow
)

from evrouting.T import Result
from evrouting import gasstation, charge
from evrouting.osm.routing import GasstationAccessFunctions

__all__ = ['gasstation_query', 'charge_query', 'classic_query', 'astar_query']


def gasstation_query(graph, conf, s, t, file):
    f = GasstationAccessFunctions(conf['consumption']['consumption_coefficient'])

    start = perf_counter()
    contracted_graph = gasstation.routing.contract_graph(
        G=graph,
        charging_stations=graph.charging_stations,
        capacity=conf['capacity'],
        f=f
    )
    contraction_time = perf_counter() - start

    start = perf_counter()
    state_graph = gasstation.routing.state_graph(
        contracted_graph,
        conf['capacity'],
        f
    )
    state_graph_time = perf_counter() - start

    start = perf_counter()
    result = gasstation.routing.shortest_path(
        G=graph,
        charging_stations=graph.charging_stations,
        s=s,
        t=t,
        initial_soc=conf['mu_s'],
        final_soc=conf['mu_t'],
        capacity=conf['capacity'],
        f=f,
        extended_graph=state_graph,
        contracted_graph=contracted_graph
    )
    query_time = perf_counter() - start

    row = GasstationQueryRow(
        start_node=s,
        target_node=t,
        query_time=query_time,
        trip_time=result.trip_time if type(result) == Result else None,
        nodes=len(graph.nodes),
        edges=len(graph.edges),
        charging_stations=len(graph.charging_stations),
        time_contracted_graph=contraction_time,
        time_state_graph=state_graph_time
    )

    write_row(file, row)


def charge_query(graph, conf, s, t, file):
    pass


def classic_query(graph, conf, s, t, file):
    pass


def astar_query(graph, conf, s, t, file):
    pass
